;********************************************************************
;
;        JJY Radio Controlled Clock display check program
;                    for display wiring check
;
;                                             Device : PIC16F877
;                                             Clock  : 12.8MHz
;                                             Author : Seiichi Inoue
;
;********************************************************************

        processor       16F877
        list            f=INHX32
        include         p16f877.inc
        radix           dec
        __config        h'3F72' ;OSC is HS
                                ;RB3,RB6,RB7 are I/O
                                ;Brown-out detection ON
                                ;Power-up timer ON
                                ;Code protection OFF
                                ;Data code protection OFF
                                ;Watchdog timer OFF
        errorlevel      -302    ;Eliminate bank warning
        errorlevel      -307    ;Eliminate Setting page bits warning

;************************  Label Definition  ************************
        cblock  h'20'
        led_ha                  ;Table head address 
        led_0                   ;7seg pattern 0
        led_1                   ;7seg pattern 1
        led_2                   ;7seg pattern 2
        led_3                   ;7seg pattern 3
        led_4                   ;7seg pattern 4
        led_5                   ;7seg pattern 5
        led_6                   ;7seg pattern 6
        led_7                   ;7seg pattern 7
        led_8                   ;7seg pattern 8
        led_9                   ;7seg pattern 9
        led_a                   ;7seg hyphen
        led_b                   ;7seg OFF
        led_10h                 ;10th of hour
        led_1h                  ;1st of hour
        led_10m                 ;10th of minute
        led_1m                  ;1st of minute
        led_10y                 ;10th of year
        led_1y                  ;1st of year
        led_10mth               ;10th of month
        led_1mth                ;1st of month
        led_10d                 ;10th of day
        led_1d                  ;1st of day
        led_10s                 ;10th of second
        led_1s                  ;1st of second
        led_wd                  ;Week
        led_cnt100u             ;100us counter
        led_wkdata              ;LED work data
        stm1                    ;Software timer 1
        stm2                    ;Software timer 2
        led_rx                  ;Rx LED
        endc

seg7_ha equ     led_0           ;LED pattern table head address

seg7_0  equ     b'00111111'     ;-gfedcba Pattern 0
seg7_1  equ     b'00000110'     ;         Pattern 1
seg7_2  equ     b'01011011'     ;         Pattern 2
seg7_3  equ     b'01001111'     ;         Pattern 3
seg7_4  equ     b'01100110'     ;         Pattern 4
seg7_5  equ     b'01101101'     ;         Pattern 5
seg7_6  equ     b'01111101'     ;         Pattern 6
seg7_7  equ     b'00000111'     ;         Pattern 7
seg7_8  equ     b'01111111'     ;         Pattern 8
seg7_9  equ     b'01101111'     ;         Pattern 9
seg7_a  equ     b'01000000'     ;         Hyphen
seg7_b  equ     b'00000000'     ;         OFF

;************************  Program Start  ***************************
        org     0               ;Reset Vector
        goto    init
        org     4               ;Interrupt Vector
        goto    int

;************************  Initial Process  *************************
init

;*** Port initialization
        bsf     status,rp0      ;Change to Bank1
        clrf    trisa           ;Set RA5-0 to OUTPUT
        bsf     trisa,4         ;Set RA4 to INPUT
        movlw   b'11111111'     ;Set input data
        movwf   trisb           ;Set RB7-0 to INPUT
        clrf    trisc           ;Set RC7-0 to OUTPUT
        clrf    trisd           ;Set RD7-0 to OUTPUT
        clrf    trise           ;Set RC2-0 to OUTPUT

;*** Set Option reg
        movlw   b'00000111'     ;PS=1:256
        movwf   option_reg      ;Set OPTION_REG

;*** ADCON1 set
        movlw   b'00000110'     ;Set all digital port
        movwf   adcon1          ;Set ADCON1
        bcf     status,rp0      ;Change to Bank0

;*** PORT set
        clrf    porta           ;Clear PORTA
        clrf    portc           ;Clear PORTC
        clrf    portd           ;Clear PORTD
        clrf    porte           ;Clear PORTE

;*** Set TMR0 reg
        movlw   d'6'            ;256-20000/80us=6
        movwf   tmr0            ;Set TMR0

;*** Display data initialization
        clrf    led_10y         ;Clear LED 10th of year
        clrf    led_1y          ;Clear LED 1st of year
        clrf    led_10mth       ;Clear LED 10th of month
        clrf    led_1mth        ;Clear LED 1st of month
        clrf    led_10d         ;Clear LED 10th of day
        clrf    led_1d          ;Clear LED 1st of day
        clrf    led_10h         ;Clear LED 10th of hour
        clrf    led_1h          ;Clear LED 1st of hour
        clrf    led_10m         ;Clear LED 10th of minute
        clrf    led_1m          ;Clear LED 1st of minute
        clrf    led_10s         ;Clear LED 10th of second
        clrf    led_1s          ;Clear LED 1st of second
        clrf    led_wd          ;Clear LED week of the day
        clrf    led_cnt100u     ;Clear LED 100us counter
        clrf    led_wkdata      ;Clear LED work data
        movlw   seg7_ha         ;Set pattern table head address
        movwf   led_ha          ;Save pattern table head address
        movlw   seg7_0          ;Set pattern 0
        movwf   led_0           ;Save pattern 0
        movlw   seg7_1          ;Set pattern 1
        movwf   led_1           ;Save pattern 1
        movlw   seg7_2          ;Set pattern 2
        movwf   led_2           ;Save pattern 2
        movlw   seg7_3          ;Set pattern 3
        movwf   led_3           ;Save pattern 3
        movlw   seg7_4          ;Set pattern 4
        movwf   led_4           ;Save pattern 4
        movlw   seg7_5          ;Set pattern 5
        movwf   led_5           ;Save pattern 5
        movlw   seg7_6          ;Set pattern 6
        movwf   led_6           ;Save pattern 6
        movlw   seg7_7          ;Set pattern 7
        movwf   led_7           ;Save pattern 7
        movlw   seg7_8          ;Set pattern 8
        movwf   led_8           ;Save pattern 8
        movlw   seg7_9          ;Set pattern 9
        movwf   led_9           ;Save pattern 9
        movlw   seg7_a          ;Set hyphen pattern
        movwf   led_a           ;Save hyphen pattern
        movlw   seg7_b          ;Set OFF pattern
        movwf   led_b           ;Save OFF pattern

;*** Display data counter set
d10h    equ     d'0'
d1h     equ     d'1'
d10m    equ     d'2'
d1m     equ     d'3'
d10y    equ     d'4'
d1y     equ     d'5'
d10mth  equ     d'6'
d1mth   equ     d'7'
d10d    equ     d'8'
d1d     equ     d'9'
d10s    equ     d'10'
d1s     equ     d'11'
dwd     equ     d'0'

        movlw   d10h            ;
        movwf   led_10h         ;Set 10th of hour counter
        movlw   d1h             ;
        movwf   led_1h          ;Set 1st of hour counter
        movlw   d10m            ;
        movwf   led_10m         ;Set 10th of minute counter
        movlw   d1m             ;
        movwf   led_1m          ;Set 1st of minute counter
        movlw   d10y            ;
        movwf   led_10y         ;Set 10th of year counter
        movlw   d1y             ;
        movwf   led_1y          ;Set 1st of year counter
        movlw   d10mth          ;
        movwf   led_10mth       ;Set 10th of month counter
        movlw   d1mth           ;
        movwf   led_1mth        ;Set 1st of month counter
        movlw   d10d            ;
        movwf   led_10d         ;Set 10th of day counter
        movlw   d1d             ;
        movwf   led_1d          ;Set 1st of day counter
        movlw   d10s            ;
        movwf   led_10s         ;Set 10th of second counter
        movlw   d1s             ;
        movwf   led_1s          ;Set 1st of second counter
        movlw   dwd             ;
        movwf   led_wd          ;Set week counter
        clrf    led_rx          ;Clear Rx LED

;*** Timer counter
        movlw   d'50'           ;Set 1 second
        movwf   stm1            ;Set software timer 1
        movlw   d'1'            ;Set second value
        movwf   stm2            ;Set software timer 2

;*** Set interrupt condition
        movlw   b'10100000'     ;GIE,TOIE,INTE=1
        movwf   intcon          ;Interruption enable

        goto    $               ;Wait interruption

int
;********** TMR0 Interruption ( 20 milliseconds interval) ***********
        bcf     intcon,t0if     ;Clear timer int flag
        movlw   d'6'            ;Set 20mS
        movwf   tmr0            ;Set TMR0

        decfsz  stm1,f          ;Software timer 1 - 1
        goto    int_end         ;Jump to interruption end
        movlw   d'50'           ;Set 1 second
        movwf   stm1            ;Set software timer 1
        decfsz  stm2,f          ;Software timer 2 - 1
        goto    int_end         ;Jump to interruption end
        movlw   d'1'            ;Set second value
        movwf   stm2            ;Set software timer 2
        goto    disp            ;Jump to display process

int_end
;****************** END of interruption process *********************
        retfie

;*** Display process ( 1 second interval )
disp
        incf    led_10h,f       ;10th of hour + 1
        movlw   d'12'           ;Set check data
        subwf   led_10h,w       ;Check over flow
        btfss   status,z        ;Over ?
        goto    disp01          ;No. Next
        clrf    led_10h         ;Clear 10th of hour
disp01
        incf    led_1h,f        ;1st of hour + 1
        movlw   d'12'           ;Set check data
        subwf   led_1h,w        ;Check over flow
        btfss   status,z        ;Over ?
        goto    disp02          ;No. Next
        clrf    led_1h          ;Clear 10th of hour
disp02
        incf    led_10m,f       ;10th of minute + 1
        movlw   d'12'           ;Set check data
        subwf   led_10m,w       ;Check over flow
        btfss   status,z        ;Over ?
        goto    disp03          ;No. Next
        clrf    led_10m         ;Clear 10th of minute
disp03
        incf    led_1m,f        ;1st of minute + 1
        movlw   d'12'           ;Set check data
        subwf   led_1m,w        ;Check over flow
        btfss   status,z        ;Over ?
        goto    disp04          ;No. Next
        clrf    led_1m          ;Clear 1st of minute
disp04
        incf    led_10y,f       ;10th of year + 1
        movlw   d'12'           ;Set check data
        subwf   led_10y,w       ;Check over flow
        btfss   status,z        ;Over ?
        goto    disp05          ;No. Next
        clrf    led_10y         ;Clear 10th of year
disp05
        incf    led_1y,f        ;1st of year + 1
        movlw   d'12'           ;Set check data
        subwf   led_1y,w        ;Check over flow
        btfss   status,z        ;Over ?
        goto    disp06          ;No. Next
        clrf    led_1y          ;Clear 1st of year
disp06
        incf    led_10mth,f     ;10th of month + 1
        movlw   d'12'           ;Set check data
        subwf   led_10mth,w     ;Check over flow
        btfss   status,z        ;Over ?
        goto    disp07          ;No. Next
        clrf    led_10mth       ;Clear 10th of month
disp07
        incf    led_1mth,f      ;1st of month + 1
        movlw   d'12'           ;Set check data
        subwf   led_1mth,w      ;Check over flow
        btfss   status,z        ;Over ?
        goto    disp08          ;No. Next
        clrf    led_1mth        ;Clear 1st of month
disp08
        incf    led_10d,f       ;10th of day + 1
        movlw   d'12'           ;Set check data
        subwf   led_10d,w       ;Check over flow
        btfss   status,z        ;Over ?
        goto    disp09          ;No. Next
        clrf    led_10d         ;Clear 10th of day
disp09
        incf    led_1d,f        ;1st of day + 1
        movlw   d'12'           ;Set check data
        subwf   led_1d,w        ;Check over flow
        btfss   status,z        ;Over ?
        goto    disp10          ;No. Next
        clrf    led_1d          ;Clear 1st of day
disp10
        incf    led_10s,f       ;10th of second + 1
        movlw   d'12'           ;Set check data
        subwf   led_10s,w       ;Check over flow
        btfss   status,z        ;Over ?
        goto    disp11          ;No. Next
        clrf    led_10s         ;Clear 10th of second
disp11
        incf    led_1s,f        ;1st of second + 1
        movlw   d'12'           ;Set check data
        subwf   led_1s,w        ;Check over flow
        btfss   status,z        ;Over ?
        goto    disp12          ;No. Next
        clrf    led_1s          ;Clear 1st of second
disp12
        incf    led_wd,f        ;Week + 1
        movlw   d'8'            ;Set check data
        subwf   led_wd,w        ;Check over flow
        btfss   status,z        ;Over ?
        goto    disp13          ;No. Next
        clrf    led_wd          ;Clear week
disp13
        movfw   led_rx          ;Read Rx LED status
        btfss   status,z        ;Rx LED OFF ?
        goto    disp14          ;No.
        incf    led_rx,f        ;Set Rx LED flag
        bsf     porta,5         ;Rx LED ON
        goto    disp15          ;Jump to interrupt end
disp14  clrf    led_rx          ;Clear Rx LED flag
        bcf     porta,5         ;Rx LED OFF
disp15
        call    led_cont        ;Display control
        goto    int_end

;*** Display control
led_cont
        movlw   b'00000011'     ;Set 10th of year selector
        movwf   porte           ;Set selector data
        movlw   b'10000000'     ;Set 10th of year selector
        iorwf   led_10y,w       ;Set data
        call    led_write       ;Write data

        movlw   b'00000010'     ;Set 1st of year selector
        movwf   porte           ;Set selector data
        movlw   b'10000000'     ;Set 1st of year selector
        iorwf   led_1y,w        ;Set data
        call    led_write       ;Write data

        movlw   b'00000001'     ;Set 10th of month selector
        movwf   porte           ;Set selector data
        movlw   b'10000000'     ;Set 10th of month selector
        iorwf   led_10mth,w     ;Set data
        call    led_write       ;Write data

        movlw   b'00000000'     ;Set 1st of month selector
        movwf   porte           ;Set selector data
        movlw   b'10000000'     ;Set 1st of month selector
        iorwf   led_1mth,w      ;Set data
        call    led_write       ;Write data

        movlw   b'00000111'     ;Set 10th of day selector
        movwf   porte           ;Set selector data
        movfw   led_10d         ;Read 10th of day
        call    led_write       ;Write data

        movlw   b'00000110'     ;Set 1st of day selector
        movwf   porte           ;Set selector data
        movfw   led_1d          ;Read 1st of day
        call    led_write       ;Write data

        movlw   b'00000011'     ;Set 10th of hour selector
        movwf   porte           ;Set selector data
        movfw   led_10h         ;Read 10th of hour
        call    led_write       ;Write data

        movlw   b'00000010'     ;Set 1st of hour selector
        movwf   porte           ;Set selector data
        movfw   led_1h          ;Read 1st of hour
        call    led_write       ;Write data

        movlw   b'00000001'     ;Set 10th of minute selector
        movwf   porte           ;Set selector data
        movfw   led_10m         ;Read 10th of minute
        call    led_write       ;Write data

        movlw   b'00000000'     ;Set 1st of minute selector
        movwf   porte           ;Set selector data
        movfw   led_1m          ;Read 1st of minute
        call    led_write       ;Write data

        movlw   b'00000100'     ;Set week selector
        movwf   porte           ;Set selector data
        movlw   b'10000000'     ;Set week selector
        iorwf   led_wd,w        ;Set data
        movwf   portd           ;Write data
        bsf     portc,5         ;Start clock pulse
        call    led_t100u       ;100usec wait
        bcf     portc,5         ;Stop clock pulse (Data latch)
        call    led_t100u       ;100usec wait

        movlw   b'00000101'     ;Set 10th of second selector
        movwf   porte           ;Set selector data
        movfw   led_10s         ;Read 10th of second
        call    led_write       ;Write data

        movlw   b'00000100'     ;Set 1st of second selector
        movwf   porte           ;Set selector data
        movfw   led_1s          ;Read 1st of second
        call    led_write       ;Data write

        return

;@@@@@@@@@@@@@@@@@@@@ Data write Subroutine @@@@@@@@@@@@@@@@@@@@@@@@@
led_write
        movwf   led_wkdata      ;Save input data
        movlw   b'00001111'     ;Set mask
        andwf   led_wkdata,w    ;Pickup LED data
        addwf   led_ha,w        ;LED pattern table HA + LED data
        movwf   fsr             ;Set table address
        movfw   indf            ;Read 7seg data
        movwf   portd           ;Set 7seg data
        movlw   b'10000000'     ;Set mask
        andwf   led_wkdata,w    ;Pick up selector data
        iorwf   portd,f         ;Set selector data
        bsf     portc,5         ;Set start clock pulse
        call    led_t100u       ;100usec wait
        bcf     portc,5         ;Set stop clock pulse (Data latch)
        call    led_t100u       ;100usec wait
        return

;@@@@@@@@@@@@@@@@@@@ 100usec Timer Subroutine @@@@@@@@@@@@@@@@@@@@@@@
led_t100u
        movlw   d'63'           ;Set loop cnt
        movwf   led_cnt100u     ;Save loop cnt
led_t100ulp
        nop                     ;Time adjust
        nop                     ;Time adjust
        decfsz  led_cnt100u,f   ;cnt100u - 1 = 0 ?
        goto    led_t100ulp     ;No, continue
        return 

;********************************************************************
;                 END of Clock display check program
;********************************************************************

       end
